@@ -339,55 +339,99 @@ class LupusecAPI:
         _LOGGER.debug("__init__.py.async_set_switch() finished.")
 
 
+    async def get_devices_async(self) -> None:
+        """ Async method to get all devices from Lupusec """
+        """ This method will call the Lupusec-API to get updated data for alle devices """
+        """ The resultset from the API-Call is stored in the cache-arrays """
+        _LOGGER.debug("__init__.py.get_devices_async called: ")
+        timeNow = time.time()
+        
+        # Get covers from cache or update from Lupusec System
+        # ToDo: update frequency shall not be hard-coded -> transfer to constant
+
+        async with aiohttp.ClientSession(auth=self._auth) as session:
+            _LOGGER.debug("auth.encode: %s", self._auth.encode())  
+
+            # Get all devices via API-CAll
+            _LOGGER.debug("__init__.py.get_devices_async(): REQUEST=%s", CONST.DEVICE_LIST_REQUEST)
+            get_devices_response = await LupusecAPI._async_api_call(self._ip_address, session, CONST.DEVICE_LIST_REQUEST)
+            _LOGGER.debug("...async_api_call(): done. check response...")
+            # Retreive Device List from Response
+            if CONST.DEVICE_LIST_HEADER in get_devices_response:
+                device_content = get_devices_response[CONST.DEVICE_LIST_HEADER]
+                print("...number of devices=", len(device_content))    
+                if (len(device_content) != 0):
+                    # empty dictionaries for various device types             
+                    binary_sensors = []
+                    switches = []
+                    covers = []
+                    # Retreive devices from device list
+                    for device in device_content:
+                        _LOGGER.debug("Device found: sid=%s, name=%s, type=%s", 
+                            device["sid"], device["name"], device["type"])
+
+                        # check on device type and add to corresponding dictionary
+                        # case: binary_sensor
+                        if (device["type"] in CONST.TYPES_BIN_SENSOR):    
+                            binary_sensors.append(device)
+                            _LOGGER.debug("device is binary sensor...added.")
+
+                        # case: switch
+                        elif (device["type"] in CONST.TYPES_SWITCH):    
+                            switches.append(device)
+                            _LOGGER.debug("device is switch...added.")
+
+                        # case: cover
+                        elif (device["type"] in CONST.TYPES_COVER):    
+                            covers.append(device)
+                            _LOGGER.debug("device is cover...added.")
+                        
+                        # case: unknwon device type
+                        else :
+                            _LOGGER.debug("unknown device type...ignoring")
+
+                    # copy results to cache dictionaries        
+                    self._cacheBinarySensors = binary_sensors
+                    self._cacheSwitches = switches
+                    self._cacheCovers = covers
+
+                else : 
+                    _LOGGER.info("ERROR: get_devices_async(): no devices found.")
+            else :
+                _LOGGER.info("ERROR: get_devices_async(): no devices found.")
+
+        _LOGGER.debug("__init__.py.get_devices_async() finished.") 
+
+
     async def get_switches_async(self) -> Dict:
-        """Async method to get switches."""
+        """ Async method to get switches. """
         _LOGGER.debug("__init__.py.get_switches_async() called: ")
         timeNow = time.time()
         
         # Get switches from cache or update from Lupusec System
         # ToDo: update frequency shall not be hard-coded -> transfer to constant
         if self._cacheSwitches is None or timeNow - self._cacheStampP > 2.0:
+            # get switches from Lupusec System
             _LOGGER.debug("...switches need update from Lupusec System.")
-            self._cacheStamp_p = timeNow
+            self._cacheStampP = timeNow
 
-            async with aiohttp.ClientSession(auth=self._auth) as session:
-                _LOGGER.debug("auth.encode: %s", self._auth.encode())  
-
-                # Get all devices and filter for switches
-                _LOGGER.debug("__init__.py.async_get_devices(): REQUEST=%s", CONST.DEVICE_LIST_REQUEST)
-                get_switches_response = await LupusecAPI._async_api_call(self._ip_address, session, CONST.DEVICE_LIST_REQUEST)
-                _LOGGER.debug("_async_api_call(): done. check response...")
-                # Retreive Device Liste from Response
-                if CONST.DEVICE_LIST_HEADER in get_switches_response:
-                    device_content = get_switches_response[CONST.DEVICE_LIST_HEADER]
-                    print("Number of devices=", len(device_content))    
-                    if (len(device_content) != 0):             
-                        switches = []
-                        for device in device_content:
-                            print("sid: ", device["sid"], ", name: ", device["name"], 
-                                ", type: ", device["type"], ", status: ", device["status"])
-                            if (device["type"] in CONST.TYPES_SWITCH):    
-                                switches.append(device)
-                                _LOGGER.debug("device is switch...added.")
-                            else :
-                                _LOGGER.debug("device is no switch...skipping.")
-                        self._cacheSwitches = switches
-                    else : 
-                        _LOGGER.info("ERROR: get_switches_async(): no switches found.")
-                else :
-                    _LOGGER.info("ERROR: get_switches_async(): no switches found.")
-
-        _LOGGER.debug("__init__.py.get_switches_async() finished.") 
+            # Make API-Call to update
+            _LOGGER.debug("...make API-call with get_devices_async() to Lupusec System.")            
+            await LupusecAPI.get_devices_async()
 
+            _LOGGER.debug("...API-call finished. check response...")
+            # Check response 
+            if (len(self._cacheSwitches) != 0):
+                _LOGGER.debug("...number of switches=%s", len(self._cacheSwitches))
+            else:
+                _LOGGER.info("ERROR: get_switches_async(): no switches found.")
+ 
+        _LOGGER.debug("get_switches_async(): checking cacheSwitches...")
         if (len(self._cacheSwitches) != 0):
-            print("Number of switches=", len(self._cacheSwitches))            
-            for switch in self._cacheSwitches:
-                print("sid: ", switch["sid"], 
-                    ", name: ", switch["name"], 
-                    ", type: ", switch["type"], 
-                    ", area: ", switch["area"],
-                    ", zone: ", switch["zone"],                     
-                    ", status: ", switch["status_ex"])
+            _LOGGER.debug("...number of switches=%s", len(self._cacheSwitches))
+        else:
+             _LOGGER.info("ERROR: get_switches_async(): no switches found.")           
+        # return dictionary with switches
         return self._cacheSwitches
 
 
@@ -400,7 +444,7 @@ class LupusecAPI:
         # ToDo: update frequency shall not be hard-coded -> transfer to constant
         if self._cacheCovers is None or timeNow - self._cacheStampP > 2.0:
             _LOGGER.debug("...covers need update from Lupusec System.")
-            self._cacheStamp_p = timeNow
+            self._cacheStampP = timeNow
 
             async with aiohttp.ClientSession(auth=self._auth) as session:
                 _LOGGER.debug("auth.encode: %s", self._auth.encode())  
@@ -598,6 +642,21 @@ class LupusecAPI:
             _LOGGER.debug("__init__.py.get_switch_async(): update from LupusecAPI finished.") 
 
 
+        # time not elapsed, Devices do not need update from Lupusec System...using Cache
+        _LOGGER.debug("__init__.py.get_switch_async(): using Cache to provide Update on Switch.")
+        if (len(self._cacheSwitches) != 0):  
+            _LOGGER.debug("__init__.py.get_switch_async(): Number of Switches=%s", len(self._cacheSwitches))             
+            for switch in self._cacheSwitches:
+                #_LOGGER.debug("__init__.py.get_switch_async(): device with sid=%s, name=%s", sensor["sid"], sensor["name"])
+                if (switch["sid"] == device_id):
+                    _LOGGER.debug("__init__.py.get_switch_async(): matching device with sid=%s found: status=%s", switch["sid"], switch["status_ex"])
+                    return switch    
+            _LOGGER.error("__init__.py.get_cover_async(): %s Switches checked, but no device found with sid=%s", len(self._cacheSwitches), device_id)
+            return {}
+        else :
+            _LOGGER.debug("ERROR: length of self._cacheSwitches = 0") 
+            return {}
+
 
     async def get_cover_async(self, device_id) -> Dict:
         """Async method to get a single cover device."""
@@ -630,8 +689,8 @@ class LupusecAPI:
                         for device in device_content:
                             if (device["type"] in CONST.TYPES_COVER):    
                                 covers.append(device)
-                                _LOGGER.debug("Cover found: sid=%s, name=%s, type=%s, status=%s", 
-                                    device["sid"], device["name"], device["type"], device["status_ex"] )
+                                _LOGGER.debug("Cover found: sid=%s, name=%s, type=%s, status=%s, onOff=%s, level=%s", 
+                                    device["sid"], device["name"], device["type"], device["status_ex"], device["onOff"], device["level"] )
                                 _LOGGER.debug("sid=%s, python_class=%s", 
                                     device["sid"], type(device["sid"]))                                    
                             else :
@@ -647,7 +706,7 @@ class LupusecAPI:
 
 
         # time not elapsed, Devices do not need update from Lupusec System...using Cache
-        _LOGGER.debug("__init__.py.get_cover_async(): using Cache to provide Update on Switch.")
+        _LOGGER.debug("__init__.py.get_cover_async(): using Cache to provide Update on Cover.")
         if (len(self._cacheCovers) != 0):  
             _LOGGER.debug("__init__.py.get_cover_async(): Number of Covers=%s", len(self._cacheCovers))             
             for cover in self._cacheCovers:
